namespace StudySystemClient {
	public class Connection {
		public signal void received(uint8[] msg);

		private SessionManager session_manager;
		private Worker worker;

		public Connection(string cert_dir) throws Error {
			var loopback = new InetAddress.loopback(SocketFamily.IPV6);
			var session_factory
				= new SessionFactory(loopback, 12888, cert_dir);
			session_manager = new SessionManager(
				session_factory, (msg) => {
					var msg_copy = new uint8[msg.length];
					Memory.copy(msg_copy, msg, msg.length);
					Idle.add(() => {
							received(msg_copy);
							return false;
						}, GLib.Priority.DEFAULT_IDLE);
				});
			worker = new Worker(session_manager);
		}

		public void send(owned uint8[] msg) {
			session_manager.send(msg);
		}
	}

	private class Worker {
		private uint TASK_PERIOD_MS = 10;

		private SessionManager session_manager;
		private bool exit;
		private Thread<void> thread;

		public Worker(SessionManager session_manager) {
			this.session_manager = session_manager;
			exit = false;
			thread = new Thread<void>("connection_worker", body);
		}

		~Worker() {
			exit = true;
			thread.join();
		}

		private void body() {
			while (!exit) {
				session_manager.task();
				Thread.usleep(1000 * TASK_PERIOD_MS);
			}
		}
	}
}
